/*
 * DimmerManager.m
 * 6/22/98 Mike Trent
 *
 * This module protects me from the crusty code surrounding managing our screen. It also puts it in one
 * place so it's easier to deal with later. It also helps keep the size of my FSController.m class down.
 */

#import "DimmerManager.h"

#ifdef GNUSTEP
#define DIMTIME 500
#define DIMSTATE 0
#endif

@implementation DimmerManager

- (id)init
{
#ifndef GNUSTEP
    if (!(eventHandle = NXOpenEventStatus())) {
        NSLog(@"NXOpenEventStatus failed -- Can't monitor screen brightness!");
        return nil;
    }

    normalBrightness = NXScreenBrightness (eventHandle);
    dimBrightness    = NXAutoDimBrightness(eventHandle);
#endif
    return self;
}

- (double)dimTime
{
#ifndef GNUSTEP
    return NXAutoDimTime(eventHandle);
#else
    /* This probably needs to hook into something in the future... */
    return DIMTIME;
#endif
}

- (void)setDimTime:(double)time
{
#ifndef GNUSTEP
    NXSetAutoDimThreshold(eventHandle, time);
#endif
}

- (BOOL)autoDimState
{
#ifndef GNUSTEP
    return NXAutoDimState(eventHandle);
#else
    return DIMSTATE;
#endif
}

- (void)brightenDimmedScreen
{
#ifndef GNUSTEP
    NXSetAutoDimBrightness(eventHandle, normalBrightness);

    NXSetAutoDimState(eventHandle, NO); // workaround?
    NXSetAutoDimState(eventHandle, YES);
#endif
}

- (void)darkenDimmedScreen
{
#ifndef GNUSTEP
    NXSetAutoDimState(eventHandle, NO);
    NXSetAutoDimBrightness(eventHandle, dimBrightness);
#endif
}


@end
