/*
 * FSController.h
 * 6/18/98 Mike Trent
 *
 * FreeSpace Controller
 */

#import <AppKit/AppKit.h>
#import "ModuleView.h"
#import "ScreenSaverWindow.h"
#import "DimmerManager.h"

enum {
    mainScreenOnly = 0,
    allScreens,
};

#define kDoesNotDim 10000000

@interface FSController : NSObject
{
    id saverBrowser;
    id settingsWindow;
    id settingsPane;
    id sleepCorner;
    id screenOption;
    id passwordSwitch;
    id passwordController;
    id aboutWindow;
    id saveTimeSlider;
    id saveTimeText;
    id pane1;
    id pane2;
    id pane3;
    id pane4;
    id tabView;
    id frameRateBtn;
    id tileView;

    // directory buttons
    id dirHomeBtn;
    id dirLocalBtn;
    id dirSystemBtn;
    id dirNetworkBtn;
    id dirBackSpaceBtn;
    id dirBackSpaceAppBtn;

    NSUserDefaults 	*defs;
    NSMutableDictionary *modules;
    NSArray		*sortedModuleNames;
    NSString		*activeModuleName;
    ModuleView 		*activeModule;
    ScreenSaverWindow 	*saverWindow;
    NSWindow		*blankWindow;
    NSTimer		*saverTimer;
    NSTimer		*cornerTimer;
    NSTimer		*dimTimer;
    NSPoint		mouse;
    int			sleepCornerState;
    int			awakeCornerState;
    BOOL		inAwakeRect;
    NSRect		sleepRect;
    NSRect		awakeRect;
    DimmerManager	*dimmerManager;
    BOOL		stoleActivation, isHidden;
    int 		screenOpt;
    NSString		*pswd;
    BOOL		appIsComingDownNow;
    //int 		dimPosition;
    int 		dimMinutes;
    int 		oldfrate;
    int 		frate;
    int			frateTime;
    int			displayFrameRate;

    // directory flags
    BOOL		dirHome;
    BOOL		dirLocal;
    BOOL		dirSystem;
    BOOL		dirNetwork;
    BOOL		dirBackSpace;
    BOOL		dirBackSpaceApp;

    BOOL		_mainScreenOnly;
    BOOL		_ignoreMouseMovement;
}
- (void)previewSaver:(id)sender;
- (void)runSaver:(id)sender;
- (void)endSaver;
- (void)selectSaver:(id)sender;
- (void)sleepCornerSet:(id)sender;
- (NSRect)setSleepRect:(int)state;
- (void)screenOptionSet:(id)sender;
- (void)passwordSwitchSet:(id)sender;
- (void)openSettingsWindow:(id)sender;
- (void)openAboutWindow:(id)sender;
- (void)saveTimeSet:(id)sender;
- (void)frameRateBtnSet:(id)sender;

// directory events
- (void)dirHomeSet:(id)sender;
- (void)dirLocalSet:(id)sender;
- (void)dirSystemSet:(id)sender;
- (void)dirNetworkSet:(id)sender;
- (void)dirBackSpaceSet:(id)sender;
- (void)dirBackSpaceAppSet:(id)sender;

- (void)awakeFromNib;
- (void)loadPrefs;
- (void)findModules;
- (void)findModulesInDirectory:(NSString*)scanDir;

- (void)loadModule:(NSString*)modName;
- (void)installModule:(ModuleView*)mod;
- (void)uninstallModule:(ModuleView*)mod;

- (void)installSaverWindow;
- (void)uninstallSaverWindow;
- (void)installBlankWindow;

- (void)installSaverTimer;
- (void)uninstallSaverTimer;
- (void)doSaverAnimation:(NSTimer*)timer;
- (void)animate:(id)sender;
- (void)installCornerTimer;
- (void)uninstallCornerTimer;
- (void)checkSleepCorner:(NSTimer)timer;
- (void)installDimTimer;
- (void)uninstallDimTimer;
- (void)checkDimState:(NSTimer)timer;

- (void)windowWillClose:(NSNotification *)aNotification;

- (int)browser:(NSBrowser *)sender numberOfRowsInColumn:(int)column;
- (void)browser:(NSBrowser *)sender willDisplayCell:(id)cell atRow:(int)row column:(int)column;
- (BOOL)browser:(NSBrowser *)sender selectRow:(int)row inColumn:(int)column;
- (void)setActiveModule:(NSString*)mod;

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename;
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification;
- (BOOL)applicationShouldTerminate:(NSApplication *)sender;

- (NSMutableDictionary*)modules;

@end

@interface AllBlackView : NSView
@end
