/*
 * FSController.m
 * 6/18/98 Mike Trent
 *
 * FreeSpace Controller
 */

#import "FSController.h"
#import "SleepCornerView.h"
#import "NSScreen+areaFrame.h"
#import "PasswordController.h"
#import "FrameRateMeter.h"

//
// Application interface for using the new about window. I'm doing this to
// supress compile-time warnings on older systems.
//

@interface NewAppClass
- (void)orderFrontStandardAboutPanelWithOptions:(NSDictionary *)optionsDictionary;
@end;

@implementation FSController

//
// Initialization
//

- (void)awakeFromNib
{
    // first hook up our tab view.
    NSTabViewItem *item;

    item = [[NSTabViewItem alloc] initWithIdentifier:nil];
    [item setView:pane1];
    [item setLabel:@"Modules"];
    [tabView addTabViewItem:item];

    item = [[NSTabViewItem alloc] initWithIdentifier:nil];
    [item setView:pane4];
    [item setLabel:@"Directories"];
    [tabView addTabViewItem:item];

    item = [[NSTabViewItem alloc] initWithIdentifier:nil];
    [item setView:pane2];
    [item setLabel:@"Screen"];
    [tabView addTabViewItem:item];

    item = [[NSTabViewItem alloc] initWithIdentifier:nil];
    [item setView:pane3];
    [item setLabel:@"Misc"];
    [tabView addTabViewItem:item];

    // set our pattern
    [tileView setImage:[NSImage imageNamed:@"SlateGray"]];

    // load the prefs.
    [self loadPrefs];

    // set our password stuff.
    [passwordController setIdleAction:@selector(animate:) forTarget:self];
    if (pswd) [passwordSwitch setState:YES];

    // set our sleepCorner stuff.
    [sleepCorner setAction:@selector(sleepCornerSet:) forTarget:self];
    [sleepCorner setPlusState:sleepCornerState];
    [sleepCorner setMinusState:awakeCornerState];
    sleepRect = [self setSleepRect:sleepCornerState];
    awakeRect = [self setSleepRect:awakeCornerState];
    [self installCornerTimer];

    // set our multi-monitor stuff.
    [screenOption selectCellWithTag:screenOpt];

    // set our timer stuff.
    [saveTimeSlider setIntValue:dimMinutes];
    if (dimMinutes == 61) [saveTimeText setStringValue:@"off"];
    else [saveTimeText setStringValue:[NSString stringWithFormat:@"%d min", dimMinutes]];

    // set our directory stuff.
    [dirHomeBtn setState:dirHome];
    [dirLocalBtn setState:dirLocal];
    [dirSystemBtn setState:dirSystem];
    [dirNetworkBtn setState:dirNetwork];
    [dirBackSpaceBtn setState:dirBackSpace];
    [dirBackSpaceAppBtn setState:dirBackSpaceApp];
    [dirBackSpaceAppBtn setEnabled:dirBackSpace];

    // set our FrameRate stuff
    [frameRateBtn setState:displayFrameRate];

    // set our window stuff.
    [settingsWindow setFrameAutosaveName:@"SettingsWindow"];
    [settingsWindow setMiniwindowImage:[NSImage imageNamed:@"FreeSpace.tiff"]];

    // do the loading.
    [self findModules];

    // set our browser stuff.
    [saverBrowser setAcceptsArrowKeys:YES];
    [saverBrowser setSendsActionOnArrowKeys:YES];
    [self setActiveModule:activeModuleName];
}

- (void)loadPrefs
{
    NSDictionary *appDefaults = [NSDictionary dictionaryWithObjectsAndKeys:
        @"General", @"activeModuleName",
        @"0",@"sleepCorner",
        @"0",@"awakeCorner",
        @"0",@"monitorSetting",
        @"YES",@"SettingsWindowOpen",
        @"NO",@"DisplayFrameRate",
        @"YES",@"DirHome",
        @"YES",@"DirLocal",
        @"YES",@"DirSystem",
        @"YES",@"DirNetwork",
        @"YES",@"DirBackSpace",
        @"YES",@"DirBackSpaceApp",
        nil];
    
    defs = [NSUserDefaults standardUserDefaults];
    [defs registerDefaults:appDefaults];

    activeModuleName = [defs stringForKey:@"activeModuleName"];
    sleepCornerState = [defs integerForKey:@"sleepCorner"];
    awakeCornerState = [defs integerForKey:@"awakeCorner"];
    screenOpt = [defs integerForKey:@"monitorSetting"];
    pswd = [[defs stringForKey:@"password"] retain];
    displayFrameRate = [[defs stringForKey:@"DisplayFrameRate"] isEqualToString:@"YES"];

    // directory switches
    dirHome = [[defs stringForKey:@"DirHome"] isEqualToString:@"YES"];
    dirLocal = [[defs stringForKey:@"DirLocal"] isEqualToString:@"YES"];
    dirSystem = [[defs stringForKey:@"DirSystem"] isEqualToString:@"YES"];
    dirNetwork = [[defs stringForKey:@"DirNetwork"] isEqualToString:@"YES"];
    dirBackSpace = [[defs stringForKey:@"DirBackSpace"] isEqualToString:@"YES"];
    dirBackSpaceApp = [[defs stringForKey:@"DirBackSpaceApp"] isEqualToString:@"YES"]; 

    // esoteric code for getting the dim time from defaults. I make no apologies for the state of this code.
    dimMinutes = ([[[defs persistentDomainForName:@"NeXT1"] objectForKey:@"AutoDimTime"] intValue] / 4080);    
}

- (void)findModules
{
    modules = [[NSMutableDictionary alloc] initWithCapacity:10];
    if (!modules) {
        NSLog(@"failed to create module list");
    }

    // look for bundles in the following places:
    [self findModulesInDirectory:[[NSBundle mainBundle] resourcePath]];
    if (dirBackSpace && dirBackSpaceApp) [self findModulesInDirectory:@"/System/Demos/BackSpace.app/Resources"];

    if (dirHome) {
        [self findModulesInDirectory:[NSString stringWithFormat:@"%@/Library/FreeSpace", NSHomeDirectory()]];
        if (dirBackSpace) [self findModulesInDirectory:[NSString stringWithFormat:@"%@/Library/BackSpace", NSHomeDirectory()]];
    }
    if (dirLocal) {
        [self findModulesInDirectory:@"/Local/Library/FreeSpace"];
        if (dirBackSpace) [self findModulesInDirectory:@"/Local/Library/BackSpace"];
    }
    if (dirSystem) {
        [self findModulesInDirectory:@"/System/Library/FreeSpace"];
        if (dirBackSpace) [self findModulesInDirectory:@"/System/Library/BackSpace"];
    }
    if (dirNetwork) {
        [self findModulesInDirectory:@"/Network/Library/FreeSpace"];
        if (dirBackSpace) [self findModulesInDirectory:@"/Network/Library/BackSpace"];
    }

    // create our module index.
    sortedModuleNames = [[[modules allKeys] sortedArrayUsingSelector:@selector(compare:)] retain];
}

- (void)findModulesInDirectory:(NSString*)scanDir
{
    NSEnumerator *e;
    NSString	 *dir;
    BOOL	isDir;

    // make sure scanDir exists and is a directory.
    if (![[NSFileManager defaultManager] fileExistsAtPath:scanDir isDirectory:&isDir]) return;
    if (!isDir) return;

    // scan the dir for .BackSpace or .FreeSpace directories.
    e = [[[NSFileManager defaultManager] directoryContentsAtPath:scanDir] objectEnumerator];
    while (dir = [e nextObject]) {
        // if that module is named "Swarm", do not load it.
        if ([dir isEqualToString:@"Swarm.BackSpace"]) continue;

        if ( ([[dir pathExtension] isEqualToString:@"BackSpace"] && dirBackSpace) ||
            [[dir pathExtension] isEqualToString:@"FreeSpace"]) {
            NSString *newKey = [dir stringByDeletingPathExtension];

            // if that module hasn't been found elsewhere, make a note of it. 
            if (![modules objectForKey:newKey]) {
                NSMutableDictionary *d = [[[NSMutableDictionary alloc] init] autorelease];
                [d setObject:[scanDir stringByAppendingPathComponent:dir] forKey:@"Path"];
                [modules setObject:d forKey:newKey];
            }
        }
    }
}


//
// Module code.
//

// loadModuleBundle
// This loads the module bundle if needed, without doing anything else.
- (void)loadModuleBundle:(NSString*)modName
{
    NSBundle	*bundle;
    Class    	viewClass;
    id	     	newModule = nil;
    NSRect	r = NSMakeRect(0.0,0.0,200.0,200.0);
    NSString 	*path;

    NSLog(@"******** Loading %@",modName);
    if (modName) {
        if (!(newModule = [[modules objectForKey:modName] objectForKey:@"Object"])) {
            path = [[modules objectForKey:modName] objectForKey:@"Path"];
            bundle = [NSBundle bundleWithPath:path];
            if (!bundle) {
                NSLog(@"Couldn't load bundle %@", path);
                return;
            }
            viewClass = [bundle principalClass];
            if (!viewClass) {
                NSLog(@"Couldn't get principalClass for %@", path);
                return;
            }
            newModule = [[viewClass alloc] initWithFrame:r];
            if (!newModule) {
                NSLog(@"Couldn't create object for %@", path);
                return;
            }
            [[modules objectForKey:modName] setObject:newModule forKey:@"Object"];
        }
    }

    NS_DURING
        if ([newModule respondsToSelector:@selector(inspector:)]) {
            [[newModule inspector:self] retain]; // send an extra retain to our inspector, so we can swap it in and out.
        }
    NS_HANDLER
        NSLog(@"%@", localException);
    NS_ENDHANDLER
      NSLog(@"************* Done........");
}

// loadModule
// This loads the module bundle and then sets the modules inspector panel.
- (void)loadModule:(NSString*)modName
{
    id	     	newModule = nil;

    if (modName) {
        [self loadModuleBundle:modName];
        newModule = [[modules objectForKey:modName] objectForKey:@"Object"]; 
    }

    if (newModule == activeModule) return;
    if (activeModule) [self uninstallModule:activeModule];

    activeModule = (ModuleView*)newModule;
    NSLog(@"Active module %@", NSStringFromClass([activeModule class]));
    [self installModule:activeModule];

    [settingsPane display];
}

- (void)installModule:(ModuleView*)mod
{
    NSView *inspectorView;

    NS_DURING
        if ([mod respondsToSelector:@selector(inspector:)]) {
            inspectorView = [mod inspector:self];

            [(NSBox*)settingsPane setBorderType:NSNoBorder];
            [(NSBox*)settingsPane setContentView:inspectorView];

            if ([mod respondsToSelector:@selector(inspectorInstalled)])
                [mod inspectorInstalled];
        }
    NS_HANDLER
        NSLog(@"%@", localException);
    NS_ENDHANDLER
}

- (void)uninstallModule:(ModuleView*)mod
{
    NS_DURING
    if ([mod respondsToSelector:@selector(inspectorWillBeRemoved)])
        [mod inspectorWillBeRemoved];
    NS_HANDLER
        NSLog(@"%@", localException);
    NS_ENDHANDLER

    [(NSBox*)settingsPane setContentView:nil];
    [(NSBox*)settingsPane setBorderType:NSGrooveBorder];
}

- (NSMutableDictionary*)modules
{
    return modules;
}

//
// Window Code
//

- (void)installSaverWindow
{
    NSRect r;
    ScreenSaverWindow *w;
    int backingStore = NSBackingStoreNonretained;

    NS_DURING
      NSLog(@"*********** INSTALLING SAVER WINDOW");
        if ([activeModule respondsToSelector:@selector(useBufferedWindow)]) {
            if ([activeModule useBufferedWindow]) backingStore = NSBackingStoreBuffered;
        }
    NS_HANDLER
        NSLog(@"%@", localException);
        backingStore = NSBackingStoreNonretained;
    NS_ENDHANDLER

    if ((screenOpt == mainScreenOnly) || _mainScreenOnly) {
        r = [[NSScreen mainScreen] frame];
    } else {
        r = [NSScreen areaFrame];
    }
        
    w = [[ScreenSaverWindow alloc] initWithContentRect:r
                                             styleMask:NSBorderlessWindowMask
                                               backing:backingStore
                                                 defer:NO];
    saverWindow = w;

    [w setAction:@selector(endSaver) forTarget:self];
    //[w setAutodisplay:NO];
    [w setAutodisplay:YES];
    [w makeFirstResponder:w];
    [w setExcludedFromWindowsMenu:YES];
    [w setBackgroundColor:[NSColor blackColor]];
    [w setOneShot:YES];
    [w setLevel:NSScreenSaverWindowLevel];
    if (backingStore == NSBackingStoreBuffered) {
        [w setDynamicDepthLimit:YES];
        [w useOptimizedDrawing:YES];
    }
    [w setAcceptsMouseMovedEvents:NO];
    if (activeModule) {
      NSLog(@"**** Setting active module view");
        [w setContentView:activeModule];

        NS_DURING
            if ([activeModule respondsToSelector: @selector(willEnterScreenSaverMode)])
                [activeModule willEnterScreenSaverMode];
        NS_HANDLER
            NSLog(@"%@", localException);
        NS_ENDHANDLER
    }
    [w makeKeyAndOrderFront:self];
    [activeModule display];
}

- (void)uninstallSaverWindow
{
    [saverWindow close]; // window will release on close.
    saverWindow = nil;
}

- (void)installBlankWindow
{
    NSRect	r;
    NSWindow 	*w;
    AllBlackView	*abv;

    r = [NSScreen areaFrame];
    abv = [[[AllBlackView alloc] initWithFrame:r] autorelease];
    w = [[NSWindow alloc] initWithContentRect:r
                                    styleMask:NSBorderlessWindowMask
                                      backing:NSBackingStoreNonretained
                                        defer:NO];
    [w setAutodisplay:NO];
    [w setOneShot:YES];
    [w setExcludedFromWindowsMenu:YES];
    [w orderFront:self];
    [w setContentView:abv];
    [abv display];
    [w setLevel:NSScreenSaverWindowLevel-1];
    blankWindow = w;
}


//
// Timer Code
//

- (void)installSaverTimer
{
    NSTimeInterval time = 0.04;

    NS_DURING
    if ([activeModule respondsToSelector:@selector(animationDelayTime)])
        time = [activeModule animationDelayTime];
    NS_HANDLER
        NSLog(@"%@", localException);
        time = 0.04;
    NS_ENDHANDLER

    saverTimer = [[NSTimer scheduledTimerWithTimeInterval:time
                                                   target:self
                                                 selector:@selector(doSaverAnimation:)
                                                 userInfo:nil
                                                  repeats:YES] retain];
}

- (void)uninstallSaverTimer
{
    if (saverTimer) {
        [saverTimer invalidate];
        [saverTimer release];
        saverTimer = nil;
    }
}

- (void)doSaverAnimation:(NSTimer*)timer
{
    NSPoint p;

    if (!saverWindow) return; // if the screen saver isn't running, don't bother.

    if (!_ignoreMouseMovement) {
        p = [saverWindow mouseLocationOutsideOfEventStream];
        p.x -= mouse.x;
        p.y -= mouse.y;

        if ((fabs(p.x) > 10.0) || (fabs(p.y) > 10.0)) {
            [self endSaver];
            return;
        }
    }

    [self animate:self];
}

- (void)animate:(id)sender
{
    int now;
    BOOL erase;
    
    if (!saverWindow) return; // if the screen saver isn't running, don't bother.

    if (!frateTime) frateTime = time(0);
    erase = ([saverWindow backingType] != NSBackingStoreBuffered);

    NS_DURING
        [activeModule lockFocus];
        if ([activeModule respondsToSelector:@selector(didLockFocus)])
            [activeModule didLockFocus];

        if (displayFrameRate) drawFrameRateMeter(oldfrate, YES);
        [activeModule oneStep];     // loop unrolling.
        if (displayFrameRate) drawFrameRateMeter(oldfrate, erase);
        [saverWindow flushWindow];
#ifndef GNUSTEP
        PSWait();
#endif
        if (displayFrameRate) drawFrameRateMeter(oldfrate, YES);
        [activeModule oneStep];
        if (displayFrameRate) drawFrameRateMeter(oldfrate, erase);
        [saverWindow flushWindow];
#ifndef GNUSTEP
        PSWait();
#endif
        [activeModule unlockFocus];
    NS_HANDLER
        NSLog(@"%@", localException);
    NS_ENDHANDLER

    frate++;
    now = time(0);
    if (frateTime != now) {
        frateTime = now;
        oldfrate = frate;
        frate=0;
    }
}

- (void)installCornerTimer
{
    if (sleepCornerState || awakeCornerState) {
        if (!cornerTimer) cornerTimer = [[NSTimer scheduledTimerWithTimeInterval:2.0
                                                                          target:self
                                                                        selector:@selector(checkSleepCorner:)
                                                                        userInfo:nil
                                                                         repeats:YES] retain];
    } else {
        if (cornerTimer) [self uninstallCornerTimer];
    }
}

- (void)uninstallCornerTimer
{
    if (cornerTimer) {
        [cornerTimer invalidate];
        [cornerTimer release];
        cornerTimer = nil;
    }
}

- (void)checkSleepCorner:(NSTimer)timer
{
    static BOOL inRect = NO, running = NO;
    static NSWindow *coordAnchor = nil;
    NSPoint pt;

    // We can't easily get global screen coordinates in AppKit. Maybe this will change as we
    // move to a QuickDraw based model? MDT
    if (!coordAnchor)
        coordAnchor = [NSWindow alloc];

    pt = [coordAnchor mouseLocationOutsideOfEventStream];

    if (NSMouseInRect(pt, awakeRect, NO)) {
        inAwakeRect = YES;
    } else {
        inAwakeRect = NO;
    }

    // see if we're in the box. If so, start saving. Try not to start the saver if we've already started it.
    if (NSMouseInRect(pt, sleepRect, NO)) {
        if (!running) {
            if (inRect) {
                [self runSaver:self];
                running = YES;
            } else {
                inRect = YES;
            }
        }
    } else {
        running = inRect = NO;
    }
}

- (void)installDimTimer
{
    dimTimer = [[NSTimer scheduledTimerWithTimeInterval:[dimmerManager dimTime]
                                                 target:self
                                               selector:@selector(checkDimState:)
                                               userInfo:nil
                                                repeats:NO] retain];
}

- (void)uninstallDimTimer
{
    if (dimTimer) {
        [dimTimer invalidate];
        [dimTimer release];
        dimTimer = nil;
    }
}

- (void)checkDimState:(NSTimer)timer
{
    [self uninstallDimTimer];

    if ([dimmerManager autoDimState]) {
        [self runSaver:self];
    } else {
        [self installDimTimer];
    }
}

//
// User Commands
//

- (void)previewSaver:(id)sender
{
}

- (void)runSaver:(id)sender
{
    if (inAwakeRect) {
        [dimmerManager brightenDimmedScreen];
        [dimmerManager darkenDimmedScreen];
        [self uninstallDimTimer];
        [self installDimTimer];
        return;
    }
    
    isHidden = [NSApp isHidden];
    
    if (![NSApp isActive]) {
        stoleActivation = YES;
        [NSApp activateIgnoringOtherApps:YES];
    } else {
        stoleActivation = NO;
    }

    // send firstCall message, which means "Someone fully intends to begin
    // the screen saver process. This is a heads-up -- you have not yet
    // been installed!" This is basically only used by Randomizer. Others
    // may also find it useful ...
    NS_DURING
        if ([activeModule respondsToSelector:@selector(firstCall)])
            [activeModule firstCall];
    NS_HANDLER
        NSLog(@"%@", localException);
    NS_ENDHANDLER

    NS_DURING
        if ([activeModule respondsToSelector:@selector(useMultipleMonitors)])
            _mainScreenOnly = ![activeModule useMultipleMonitors];
        else
            _mainScreenOnly = NO;
    NS_HANDLER
        NSLog(@"%@", localException);
        _mainScreenOnly = NO;
    NS_ENDHANDLER

    // MDT 1/9/2000 For BackSpace compatability
    NS_DURING
        if ([activeModule respondsToSelector:@selector(ignoreMouseMovement)])
            _ignoreMouseMovement = [activeModule ignoreMouseMovement];
        else
            _ignoreMouseMovement = NO;
    NS_HANDLER
        NSLog(@"%@", localException);
        _ignoreMouseMovement = NO;
    NS_ENDHANDLER

// I debug on a multi-monitor system. This allows me to run the saver on one
// screen while debugging away on the other one.
#ifndef DEBUG
    [NSCursor hide];
#else
    screenOpt = mainScreenOnly;
#endif
    [self installSaverWindow];
#ifndef DEBUG
    if ((screenOpt == mainScreenOnly) || _mainScreenOnly)
        [self installBlankWindow];
#endif
    [self installSaverTimer];
    [self uninstallCornerTimer];
    [self uninstallDimTimer];
    [dimmerManager brightenDimmedScreen];
    mouse = [saverWindow mouseLocationOutsideOfEventStream];

    NS_DURING
        if ([activeModule respondsToSelector:@selector(enteredScreenSaverMode)])
            [activeModule enteredScreenSaverMode];
    NS_HANDLER
        NSLog(@"%@", localException);
    NS_ENDHANDLER

    frate = oldfrate = frateTime = 0;
}

- (void)endSaver
{
    if (pswd) {
        BOOL ok;
        [NSCursor unhide];
        ok = [passwordController validatePassword:pswd
                                          timeOut:30.0
                                   randomPosition:YES];
        [NSCursor hide];
        if (!ok) {
            mouse = [saverWindow mouseLocationOutsideOfEventStream];
            return;
        }
    }

    NS_DURING
        if ([activeModule respondsToSelector: @selector(willExitScreenSaverMode)])
            [activeModule willExitScreenSaverMode];
    NS_HANDLER
        NSLog(@"%@", localException);
    NS_ENDHANDLER

    [NSCursor unhide];
    if (isHidden) [NSApp hide:self];
    if (stoleActivation) [NSApp deactivate];
    [dimmerManager darkenDimmedScreen];
#ifndef DEBUG
    if ((screenOpt == mainScreenOnly) || _mainScreenOnly) [blankWindow close];
#endif
    [self uninstallSaverWindow];
    [self uninstallSaverTimer];
    [self installCornerTimer];
    [self installDimTimer];
}

- (void)selectSaver:(id)sender
{
    id  mod = nil;
    int row = [saverBrowser selectedRowInColumn:[saverBrowser selectedColumn]];
    
    if (row >= 0) mod = [sortedModuleNames objectAtIndex:row];
    [self loadModule:mod];
    //activeModuleName = mod;

    [defs setObject:mod forKey:@"activeModuleName"];
}

- (void)sleepCornerSet:(id)sender
{
    sleepCornerState = [sleepCorner plusState];
    awakeCornerState = [sleepCorner minusState];

    sleepRect = [self setSleepRect:sleepCornerState];
    awakeRect = [self setSleepRect:awakeCornerState];
    [self installCornerTimer];
    
    [defs setObject:[NSString stringWithFormat:@"%d", sleepCornerState] forKey:@"sleepCorner"];
    [defs setObject:[NSString stringWithFormat:@"%d", awakeCornerState] forKey:@"awakeCorner"];
}

-(NSRect)setSleepRect:(int)state
{
    // MDT 1/9/2000 Just pin this to the actual visible area. This may have
    // undesired side-effects, but it's better than having to invent a
    // sophisticated display manager UI. For now.
    // NSRect screen = [[NSScreen mainScreen] frame];
    NSRect screen = [NSScreen areaFrame];
    NSRect result;

    result.size = NSMakeSize(4,4);
    switch(state) {
        case 0: // bogus
            result.origin = NSMakePoint(-9,-9);
            break;
        case 1: // lower left
            result.origin = NSMakePoint(-1,-1);
            break;
        case 2: // upper left
            result.origin = NSMakePoint(-1,screen.size.height-3);
            break;
        case 3: // upper right
            result.origin = NSMakePoint(screen.size.width-3,screen.size.height-3);
            break;
        case 4: // lower right
            result.origin = NSMakePoint(screen.size.width-3,-1);
            break;
    }
    
    return result;
}

- (void)screenOptionSet:(id)sender
{
    screenOpt = [[sender selectedCell] tag];

    [defs setObject:[NSString stringWithFormat:@"%d", screenOpt] forKey:@"monitorSetting"];
}

- (void)passwordSwitchSet:(id)sender;
{
    BOOL abort=NO;
    NSString *new;
    
    if ( [passwordController validatePassword:pswd timeOut:0 randomPosition:NO] ) {

        if ([sender state]) {
            // we're turning password checking on
            new = [passwordController setPassword];
            if (new) {
                [pswd release];
                pswd = new;

                [defs setObject:pswd forKey:@"password"];

            } else abort = YES;
        } else {
            // we're turning password checking off
            [pswd release];
            pswd = nil;

            [defs removeObjectForKey:@"password"];
        }
    } else abort = YES;

    if (abort) [sender setState:![sender state]];
}

// Specialized code for saving our dim time default. This is all pretty esoteric, to keep compatability
// with Preferences.app, NS 3.3, and probably half a dozen other things ... 
- (void)saveTimeSet:(id)sender
{
    int val;
    NSMutableDictionary *d;
    NSString *next1 = @"NeXT1";

    val = dimMinutes = [saveTimeSlider intValue];
    if (val == 61) {
        [saveTimeText setStringValue:@"off"];
        val = kDoesNotDim;
    } else {
        [saveTimeText setStringValue:[NSString stringWithFormat:@"%d min", val]];
        val *= 60;
    }
    [dimmerManager setDimTime:val];

    if (val != kDoesNotDim) val *= 68;
    d = [NSMutableDictionary dictionaryWithDictionary:[defs persistentDomainForName:next1]];
    [d setObject:[NSString stringWithFormat:@"%d", val] forKey:@"AutoDimTime"];
    [defs removePersistentDomainForName:next1];
    [defs setPersistentDomain:d forName:next1];

    [self uninstallDimTimer];
    [self installDimTimer];
}

- (void)openSettingsWindow:(id)sender
{
    [settingsWindow makeKeyAndOrderFront:sender];
    [defs setObject:@"YES" forKey:@"SettingsWindowOpen"];
}

- (void)openAboutWindow:(id)sender
{
    // On older systems, orderFrontStandardAboutPanelWithOptions doesn't exist.
    // In that case, we'll continue showing the old about panel. Otherwise,
    // we'll use the new AppKit-supplied one.
    if ([NSApp respondsToSelector:@selector(orderFrontStandardAboutPanelWithOptions:)]) {
        NSDictionary *options;

        options = [NSDictionary dictionaryWithObjectsAndKeys:
            @"1.3",	@"Version",
            @"Copyright  1998-2000 Michael D. Trent", @"Copyright",
            [NSImage imageNamed:@"FreeSpace.tiff"], @"ApplicationIcon",
            nil];

        [(NewAppClass*)NSApp orderFrontStandardAboutPanelWithOptions:options];
    } else {
        [aboutWindow center];
        [aboutWindow makeKeyAndOrderFront:sender];
    }
}

- (void)frameRateBtnSet:(id)sender
{
    displayFrameRate = [frameRateBtn state];
    [defs setObject:(displayFrameRate ? @"YES" : @"NO") forKey:@"DisplayFrameRate"];
}

// Directory events.

- (void)dirHomeSet:(id)sender
{
    dirHome = [dirHomeBtn state];
    [defs setObject:(dirHome ? @"YES" : @"NO") forKey:@"DirHome"];
}

- (void)dirLocalSet:(id)sender
{
    dirLocal = [dirLocalBtn state];
    [defs setObject:(dirLocal ? @"YES" : @"NO") forKey:@"DirLocal"];
}

- (void)dirSystemSet:(id)sender
{
    dirSystem = [dirSystemBtn state];
    [defs setObject:(dirSystem ? @"YES" : @"NO") forKey:@"DirSystem"];
}

- (void)dirNetworkSet:(id)sender
{
    dirNetwork = [dirNetworkBtn state];
    [defs setObject:(dirNetwork ? @"YES" : @"NO") forKey:@"DirNetwork"];
}

- (void)dirBackSpaceSet:(id)sender
{
    dirBackSpace = [dirBackSpaceBtn state];
    [defs setObject:(dirBackSpace ? @"YES" : @"NO") forKey:@"DirBackSpace"];

    [dirBackSpaceAppBtn setEnabled:dirBackSpace];
}

- (void)dirBackSpaceAppSet:(id)sender
{
    dirBackSpaceApp = [dirBackSpaceAppBtn state];
    [defs setObject:(dirBackSpaceApp ? @"YES" : @"NO") forKey:@"DirBackSpaceApp"];
}


//
// NSWindow delegate methods
//

- (void)windowWillClose:(NSNotification *)aNotification
{
    if (appIsComingDownNow) return;
    
    if ([aNotification object] == settingsWindow) {
        [defs setObject:@"NO" forKey:@"SettingsWindowOpen"];
    }
}

//
// NSBrowser Delegate methods
//

- (int)browser:(NSBrowser *)sender numberOfRowsInColumn:(int)column
{
    return [sortedModuleNames count];
}

- (void)browser:(NSBrowser *)sender willDisplayCell:(id)cell atRow:(int)row column:(int)column
{
    [cell setStringValue:[sortedModuleNames objectAtIndex:row]];
    [cell setLeaf:YES];
}

- (BOOL)browser:(NSBrowser *)sender selectRow:(int)row inColumn:(int)column
{
    return YES; 
}

- (void)setActiveModule:(NSString*)mod
{
    if (![modules objectForKey:mod]) mod = @"General";
    [self loadModule:mod];
    [saverBrowser loadColumnZero];
    [saverBrowser setPath:mod];
}

//
// NSApplication Delegate methods
//

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
    NSString *name;

    if ([[filename pathExtension] isEqualToString:@"BackSpace"] || [[filename pathExtension] isEqualToString:@"FreeSpace"]) {
        name = [[filename lastPathComponent] stringByDeletingPathExtension];

        // if that module hasn't been found elsewhere, make a note of it.
        if (![modules objectForKey:name]) {
            NSMutableDictionary *d = [[[NSMutableDictionary alloc] init] autorelease];
            [d setObject:filename forKey:@"Path"];
            name = [[filename lastPathComponent] stringByDeletingPathExtension];
            [modules setObject:d forKey:name];

            [sortedModuleNames release];
            sortedModuleNames = [[[modules allKeys] sortedArrayUsingSelector:@selector(compare:)] retain];
            [self setActiveModule:name];

            return YES;
        }
    }
    return NO;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Install a timer that checks for our hot corner to be set.
    [self installCornerTimer];

    // be polite and set the random seed once, at the top.
    srandom(time(0));

    // init our dimmer manager and get the ball rolling.
    dimmerManager = [[DimmerManager alloc] init];
    [self installDimTimer];

    // if the settings window should be opened automatically, open it now.
    if ([[defs objectForKey:@"SettingsWindowOpen"] isEqualToString:@"YES"])
        [settingsWindow makeKeyAndOrderFront:self];       

    // if we autolaunched, hide ourselves.
    if ([defs objectForKey:@"NXAutoLaunch"] || [defs boolForKey:@"NSAutoLaunch"]) {
        [NSApp hide:self];
    }
}

- (BOOL)applicationShouldTerminate:(NSApplication *)sender
{
    // tell WorkSpace to shut us down if we can clear the screensaver.
    if (saverWindow) {
        [self endSaver];
        if (saverWindow) return NO; // we're password locked. This could use better API.
    }

    appIsComingDownNow = YES;
    
    return YES;
}

@end

@implementation AllBlackView

- (void)drawRect:(NSRect)r
{
    [[NSColor blackColor] set];
    NSRectFill(r);
}

@end

