/*
 * GeneralView.m
 * 6/29/98 Mike Trent
 *
 * loginwindow.app-style screen saver.
 *
 * 8/10/98  MDT added setFrameSize to better deal with BackSpace.
 */

// defined by FreeSpace.
float randBetween(float a, float b);

#import "GeneralView.h"

#ifndef GNUSTEP
#import <libc.h>
#endif

@implementation StaticGeneralView

- (void)drawRect:(NSRect)rects
{
    NSRect r,bounds;
    int saftey=0;

    NSLog(@"********* Drawing (Static)");
    // clip to our rect.
    NSRectClip(rects);

    [[NSColor blackColor] set];
    NSRectFill(rects);

    bounds = [self bounds];
    NSLog(@"*************** Bounds of View: %@", NSStringFromRect(bounds));
    r = NSMakeRect(bounds.origin.x - (bounds.size.width/2),
                   bounds.origin.y - (bounds.size.height/2),
                   bounds.size.width * 2,
                   bounds.size.height* 2);
    NSLog(@"******** r = %@", NSStringFromRect(r));

    do {
      //NSLog(@"********** Moving image");
        [self moveImage:r];
        [self moveHostname:r];
        if (saftey++ > 100) break;
    } while ( (NSIsEmptyRect(NSIntersectionRect([self bounds] , imageFrame))) ||
              (NSIsEmptyRect(NSIntersectionRect([self bounds] , textFrame))) );

    [image compositeToPoint:imageFrame.origin operation:NSCompositeSourceOver];
    [hostname drawInteriorWithFrame:textFrame inView:self];
    NSLog(@"Leaving drawRect method");
    fprintf(stderr, "********** imageFrame = %s, textFrame = %s", 
	  [NSStringFromRect(imageFrame) cString], 
	  [NSStringFromRect(textFrame) cString]);
}

@end

@implementation GeneralView

- (id)initWithFrame:(NSRect)frameRect
{
    NSDictionary *defaults;
    NSString *imageFileName, *host;
    NSFileManager *fm;
    char hname[MAXHOSTNAMELEN];

    if (![super initWithFrame:frameRect]) return nil;
    srandom(time(0));
    fm = [NSFileManager defaultManager];

    NSLog(@"Initializing....");

    // load image based on loginwindow.app heuristics.
#ifndef GNUSTEP
    defaults = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"loginwindow"];
    if ([fm fileExistsAtPath:[defaults objectForKey:@"ScreenSaverImage"]]) {
        imageFileName = [defaults objectForKey:@"ScreenSaverImage"];
    } else if ([fm fileExistsAtPath:@"/System/Library/CoreServices/loginwindow.app/Resources/nexthalo.tiff"]) {
        imageFileName = @"/System/Library/CoreServices/loginwindow.app/Resources/nexthalo.tiff";
    } else if ([fm fileExistsAtPath:@"/System/Library/CoreServices/loginwindow.app/Resources/ScreenSaver.tiff"]) {
        imageFileName = @"/System/Library/CoreServices/loginwindow.app/Resources/ScreenSaver.tiff";
    } else {
        imageFileName = [[NSBundle bundleForClass:[self class]] pathForImageResource:@"apple"];
    }
#else
    if ([fm fileExistsAtPath:@"/home/heron/Images/MOSXLoginPanel.tiff"])
      {
	NSLog(@"Setting up image");
	imageFileName = @"/home/heron/Images/MOSXLoginPanel.tiff";
      }
#endif

    image = [[NSImage alloc] initWithContentsOfFile:imageFileName];
    imageFrame.size = [image size];
    NSLog(@"******** imageFile.size = %@", NSStringFromSize(imageFrame.size));

    // divine our hostname & create our cell.
    (void)gethostname(hname,MAXHOSTNAMELEN);
    host = [NSString stringWithCString:hname];
    hostname = [[NSTextFieldCell allocWithZone:[self zone]] initTextCell:host];
    [hostname setBordered:NO];
    [hostname setBezeled:NO];
    [hostname setDrawsBackground:NO];
    [hostname setWraps:NO];
    [hostname setFont:[NSFont fontWithName:@"Times-Italic" size:36.0]];
    [hostname setTextColor:[NSColor darkGrayColor]];
    [hostname setBackgroundColor:[NSColor blackColor]];
    textFrame.size = [hostname cellSize];

    return self;
}

- (void)drawRect:(NSRect)rects
{
    [[NSColor blackColor] set];
    NSRectFill(rects);
    
    NSLog(@"********* Drawing");
    // if we have obscured our image or our string, redraw them.
    if (!timer) return; // we haven't really set our rects yet; Leave it blank.
    if (!NSIsEmptyRect(NSIntersectionRect(rects , imageFrame))) {
      NSLog(@"******** Rendering image1");
        [image compositeToPoint:imageFrame.origin operation:NSCompositeSourceOver];
    }
    if (!NSIsEmptyRect(NSIntersectionRect(rects , textFrame))) {
      NSLog(@"******** Rendering image2");
        [hostname drawInteriorWithFrame:textFrame inView:self];
    }
}

- (void)setFrameSize:(NSSize)newSize
{
    [super setFrameSize:newSize];
    [self moveImage:[self bounds]];
    [self moveHostname:[self bounds]];
}

- (NSTimeInterval)animationDelayTime
{
    return 1.0;
}

- (void)oneStep
{
    if ( (timer) && ([timer compare:[NSDate date]] != NSOrderedAscending) ) return;
    
    [timer release];
    timer = [[NSDate dateWithTimeIntervalSinceNow:120.0] retain];

    [self moveImage:[self bounds]];
    [self moveHostname:[self bounds]];
    [self drawRect:[self bounds]];
}

- (void)moveImage:(NSRect)r
{
  //imageFrame.origin = [self randomPointForSize:imageFrame.size within:r];
  imageFrame.origin.x = 1000;
  imageFrame.origin.y = 1000;
}

- (void)moveHostname:(NSRect)r
{
    int i=0;

    // pick a random point for our textField rect.
    textFrame.origin = [self randomPointForSize:textFrame.size within:r];
    textFrame.origin.x = 800;
    textFrame.origin.y = 800;
    return;

    // if the new rect overlapps our image rect, try again.
    while (!NSIsEmptyRect(NSIntersectionRect(textFrame , imageFrame))) {
        textFrame.origin = [self randomPointForSize:textFrame.size within:r];

        i++;
        if (i > 50) break;
    }

    // Saftey net: If we still haven't gotten it right, just pick a "best guess".
    // This should only happen on the smallest of monitors, and in our static view.
    if (i > 50) {
        int x,y,maxx,maxy;
        if (((NSMinX(imageFrame) + NSWidth(imageFrame)/2) < (NSWidth(r) / 2))) {
            x = NSMaxX(imageFrame);
            maxx = NSWidth(r);
        } else {
            x = 0;
            maxx = NSMinX(imageFrame);
        }
        if (((NSMinY(imageFrame) + NSHeight(imageFrame)/2) < (NSHeight(r) / 2))) {
            y = NSMaxY(imageFrame);
            maxy = NSHeight(r);
        } else {
            y = 0;
            maxy = NSMinY(imageFrame);
        }
        if (((maxx - x) > NSWidth(textFrame)) && ((maxy - y) > NSHeight(textFrame))){
            textFrame.origin.x = x;
            textFrame.origin.y = y;
        }
    }
}

- (NSPoint)randomPointForSize:(NSSize)size within:(NSRect)bounds
{
    NSPoint pt;

    pt.x = floor(randBetween(bounds.origin.x, bounds.size.width - size.width));
    pt.y = floor(randBetween(bounds.origin.y, bounds.size.height - size.height));
 
    //NSLog(@"********* Random point %@ from within bounds %@", 
    //  NSStringFromPoint(pt), 
    //  NSStringFromRect(bounds));
    
    return pt;
}

- (void)willExitScreenSaverMode
{
    [timer release];
    timer = nil;
}

- (NSView *)inspector:sender
{
    if (!sharedInspectorPanel) {
        [NSBundle loadNibNamed:@"General.nib" owner:self];
    }

    return sharedInspectorPanel;
}

@end
