/*
 * PasswordController.m
 * 6/23/98 Mike Trent
 *
 * Password Controller for dealing with password protected screen savers.
 */

#import "PasswordController.h"
#import "FSUtil.h"  // for randBetween
#include <unistd.h> // for crypt

static char * salt = "Uh";

@implementation PasswordController

- (id)init
{
    [super init];

    [window setLevel:NSScreenSaverWindowLevel];
    [window setExcludedFromWindowsMenu:YES];

    return self;
}

- (void)enter:(id)sender
{
    [NSApp stopModal];
}

- (BOOL)validatePassword:(NSString*)epw timeOut:(NSTimeInterval)time randomPosition:(BOOL)rp
{
    NSDate 		*timeOut;
    NSModalSession	session;
    BOOL		result;
    BOOL		weDidntTimeOut=YES;

    if (!epw) return YES; // if there is no password, we automatically validate. This is taken advantage of in FSController
    
    [prompt setTextColor:[NSColor blackColor]];
    [prompt setStringValue:@"Enter password:"];
    session = [NSApp beginModalSessionForWindow:window];

    // position our window.
    if (rp) {
        NSRect              r;
        NSSize              s;
        NSPoint             p;

        s = [[NSScreen mainScreen] frame].size;
        r = [window frame];
        p.x = floor(randBetween(0, s.width - r.size.width));
        p.y = floor(randBetween(0, s.height - r.size.height));

        [window setFrameOrigin:p];
    } else {
        [window center];
    }
    
    timeOut = [NSDate dateWithTimeIntervalSinceNow:time];
    [field selectText:self];
    [window setLevel:NSScreenSaverWindowLevel]; // for some reason, I need to remind the window of it's level.
    [window makeKeyAndOrderFront:self];

    // get the password
    do {
        if ([NSApp runModalSession:session] != NSRunContinuesResponse)
            break;

        [target animate:self];

        weDidntTimeOut = ( ([timeOut compare:[NSDate date]] != NSOrderedAscending) || (time <= 0) );
    } while (weDidntTimeOut);
    [NSApp endModalSession:session];

    // if we didn't time out, make sure the passwords match
    if (weDidntTimeOut) {
        NSString *pw;

        pw = [NSString stringWithCString:crypt([[field stringValue] cString], salt)];
        [field setStringValue:@""];
        if ([pw isEqualToString:epw]) {
            result = YES;
        } else {
            result = NO;

            [prompt setTextColor:[NSColor redColor]];
            [prompt setStringValue:@"Incorrect password"];
            [window setLevel:NSScreenSaverWindowLevel]; // for some reason, I need to remind the window of it's level.
            [prompt display]; // and for some other reason, I really need to force redrawing of both the prompt
            [window display]; // and the window. :-P
            // we pulled SoundKit for DR2 ... I won't use a sound until NSSound ships.
            
            timeOut = [NSDate dateWithTimeIntervalSinceNow:2.0];
            while ([timeOut compare:[NSDate date]] != NSOrderedAscending) [target animate:self];
        }
    } else {
        result = NO;
    }
    [window orderOut:self];
    return result;
}

- (NSString*)setPassword
{
    NSString 		*a, *b, *result;

    [prompt setTextColor:[NSColor blackColor]];
    [prompt setStringValue:@"Enter a password:"];
    [field setStringValue:@""];
    [field selectText:self];
    [window center];
    [window makeKeyAndOrderFront:self]; 

    [NSApp runModalForWindow:window];
    a = [NSString stringWithCString:crypt([[field stringValue] cString], salt)];
    [field setStringValue:@""];

    [prompt setStringValue:@"Confirm password:"];
    [field selectText:self];

    [NSApp runModalForWindow:window];
    b = [NSString stringWithCString:crypt([[field stringValue] cString], salt)];
    [field setStringValue:@""];
    
    if ([a isEqualToString:b]) {
        result = [a retain];
    } else {
        NSDate *timeOut;
        [prompt setTextColor:[NSColor redColor]];
        [prompt setStringValue:@"Passwords don't match"];
        [prompt display];
        [window display];
        // we pulled SoundKit for DR2 ... I won't use a sound until NSSound ships.
        
        timeOut = [NSDate dateWithTimeIntervalSinceNow:2.0];
        while ([timeOut compare:[NSDate date]] != NSOrderedAscending);

        result = nil;
    }

    [window orderOut:self];
    return result;
}

// this is used for spinning the screen saver when the window is up.
- (void)setIdleAction:(SEL)a forTarget:(id)t
{
    target = t;
    action = a;
}


@end
