/*
 * ScreenSaverWindow.m
 * 6/22/98 Mike Trent
 *
 * Screen Saver Window code. This allows us to wake from key-down events. Make sure the
 * window is FirstResponder.
 */


#import "ScreenSaverWindow.h"

@implementation ScreenSaverWindow

- (void)setAction:(SEL)theAction forTarget:(id)theTarget
{
    action = theAction;
    target = theTarget;
}

- (void)keyDown:(NSEvent *)theEvent
{
    [NSApp sendAction:action to:target from:self];
}

- (void)flagsChanged:(NSEvent *)theEvent
{
    // I had to add the test for non-zero modifier flags to cover the case where the user
    // invokes the screen saver from the menu. If they press command-R to run the saver, the
    // system notices they took their finger off of the command key and removes the screen
    // saver window after 4 frames or so. This allows the modifier key (and any other modifier)
    // to turn off without waking the saver. 
    if ([theEvent modifierFlags]) [NSApp sendAction:action to:target from:self];
}

- (void)mouseUp:(NSEvent*)theEvent
{
    [NSApp sendAction:action to:target from:self];
}

- (BOOL)canBecomeKeyWindow
{
    return YES;
}

- (BOOL)canBecomeMainWindow
{
    return YES;
}

@end
