/*
 * SleepCornerView.m
 * 6/22/98 Mike Trent
 *
 * Sleep Corner selection code. It's easier to subclass NSView and do all our work in mouseDown: than it is
 * to subclass NSControl and NSCell. This means we can't hook things up in IB, so I have supplied
 * -(void)setAction:forTarget: to abstractly handle sending messages back to our controller. 
 */

#import "SleepCornerView.h"

static NSPoint cornerOrigins[] = {{0,0}, {6,13}, {6,28}, {29,28}, {29,13}};

@implementation SleepCornerView

- (id)initWithFrame:(NSRect)frameRect
{
    [super initWithFrame:frameRect];
    
    imacImage = [NSImage imageNamed:@"imac.tiff"];
    if (!imacImage) {
        NSLog(@"failed to load imac.tiff");
        return nil;
    }
    plusImage = [NSImage imageNamed:@"plus.tiff"];
    if (!plusImage) {
        NSLog(@"failed to load plus.tiff");
        return nil;
    }
    minusImage = [NSImage imageNamed:@"minus.tiff"];
    if (!minusImage) {
        NSLog(@"failed to load minus.tiff");
        return nil;
    }

    return self;
}

- (void)drawRect:(NSRect)rects
{
    NSPoint zero = {0.0,0.0};

    [imacImage compositeToPoint:zero operation:NSCompositeCopy];
    if (plusState) {
        [plusImage compositeToPoint:cornerOrigins[plusState] operation:NSCompositeCopy];
    }
    if (minusState) {
        [minusImage compositeToPoint:cornerOrigins[minusState] operation:NSCompositeCopy];
    }
}

- (int)plusState
{
    return plusState;
}

- (void)setPlusState:(int)value
{
    plusState = value;
    [self display];
}

- (int)minusState
{
    return minusState;
}

- (void)setMinusState:(int)value
{
    minusState = value;
    [self display];
}


- (void)mouseDown:(NSEvent*)theEvent
{
    NSPoint p; 
    NSRect r = { {0,0}, {13,13} };
    int i,tmp=-1;
    BOOL opt;
    
    p = [theEvent locationInWindow];
    p = [self convertPoint:p fromView:nil];
    opt = ([theEvent modifierFlags] & NSAlternateKeyMask) ? YES : NO;

    for (i = 1; i <= 4; i++) {
        r.origin = cornerOrigins[i];

        if (NSPointInRect(p,r)) {
            tmp = i;
            break;
        }
    }

    if (tmp != -1) {
        // we clicked a hot-square.
        if ( (tmp == plusState) && (!opt) ) {  // we clicked on the plus, turning the plus off.
            plusState = 0;
        } else if ( (tmp == minusState) && (opt) ) { // we opt-clicked on the minus, turning the minus off
            minusState = 0;
        } else {
            if (opt) minusState = tmp;
            else plusState = tmp;

            if (minusState == plusState) {
                if (opt) plusState = 0;     // we moved the minus onto the plus
                else minusState = 0;        // we moved the plus onto the minus
            }
        }
    }

    [self display];
    [NSApp sendAction:action to:target from:self];

}

- (void)setAction:(SEL)theAction forTarget:(id)theTarget
{
    action = theAction;
    target = theTarget;
}

@end
