/*
 * SpotLightView.m
 * 6/30/98 Mike Trent
 *
 * SpotLight screen saver module.
 */

#import "SpotLightView.h"

#ifdef GNUSTEP
#import <AppKit/PSOperators.h>
#import <math.h>
#else
#import <AppKit/psopsNeXT.h>
#endif

static NSString *speedStrings[] = {
    @"Largo",
    @"Adagio",
    @"Andante",
    @"Allegro",
    @"Presto",
};

static NSString *sizeStrings[] = {
    @"Small",
    @"Medium",
    @"Large",
    @"Extra Large",
    @"Super Sized",
};

// defined by FreeSpace
float randBetween(float a, float b);

/*static float randomSign(void)
{
    return ( (random() - (RAND_MAX/2)) > 0 ) ? 1.0 : -1.0;
}*/

static NSRect InsetRect(NSRect r, float x, float y)
{
    r.origin.x += x;
    r.origin.y += y;
    r.size.width -= x+x;
    r.size.height -= y+y;
    return r;
}

@implementation SpotLightView

//
// Initialization
//

- (BOOL)useBufferedWindow
{
    return YES;
}

- (id)initWithFrame:(NSRect)frameRect
{
    return [self initWithFrame:frameRect loadInspectorPanel:YES];
}

- (id)initWithFrame:(NSRect)frameRect loadInspectorPanel:(BOOL)loadPanel;
{
    if (![super initWithFrame:frameRect]) return nil;

    [self allocateGState]; 
    srandom(time(0));

    [self loadDefaults];
    
    if (loadPanel) {
        (void)[self inspector:self];

        [transparentBox setState:transparent];
        [self transparentSet:self];
        [sizeSlider setIntValue:size];
        [self sizeSet:self];
        [speedSlider setIntValue:speed];
        [self speedSet:self];
        [spotSlider setIntValue:numSpots];
        [self spotSet:self];
    }

    spotRect = NSMakeRect(0,0,40.0 * size, 40.0 * size);
    [self setSpotList:[self bounds]];

    return self;
}

- (void)setSpotList:(NSRect)r
{
    int i;

    midx = ([self bounds].size.width - spotRect.size.width) / 2;
    midy = ([self bounds].size.height - spotRect.size.height) / 2;
    
    for (i = 0; i < numSpots; i++) {
        /*spots[i].rect.size = spotRect.size;
        spots[i].rect.origin = [self randomPointForSize:(spotRect.size) within:r];
        spots[i].vector.x = speed * randomSign();
        spots[i].vector.y = speed * randomSign();*/

        spots[i].vector.x = randBetween(0,(2*M_PI));
        spots[i].vector.y = randBetween(0,(2*M_PI));
        spots[i].rect.size = spotRect.size;
        spots[i].rect.origin.x = cos(spots[i].vector.x) * midx + midx;
        spots[i].rect.origin.y = sin(spots[i].vector.y) * midy + midy;
    }
}

- (void)loadDefaults
{
    size = [[[NSUserDefaults standardUserDefaults] objectForKey:@"SpotLightSize"] intValue];
    if ( (size < 1) || (size > 5) ) size = 3;
    
    speed = [[[NSUserDefaults standardUserDefaults] objectForKey:@"SpotLightSpeed"] intValue];
    if ( (speed < 1) || (speed > 5) ) speed = 3;
    
    numSpots = [[[NSUserDefaults standardUserDefaults] objectForKey:@"SpotLightSpots"] intValue];
    if (numSpots > kMaxSpots) numSpots = kMaxSpots;
    if (numSpots == 0) numSpots = 3;
    if (numSpots < 0) numSpots = 1;

    transparent = [[[NSUserDefaults standardUserDefaults] objectForKey:@"SpotLightTrans"] isEqualToString:@"YES"];
}

- (NSView *)inspector:sender
{
    if (!sharedInspectorPanel) {
        [NSBundle loadNibNamed:@"SpotLight.nib" owner:self];
    }

    return sharedInspectorPanel;
}

//
// The Graphics Code
//

- (void)drawRect:(NSRect)rects
{    
    [[NSColor blackColor] set];
    NSRectFill(rects);
}

- (void)oneStep
{
    int i;

    // if five minutes have gone by, rearrange the spots.
    if ( (!timer) || ([timer compare:[NSDate date]] == NSOrderedAscending) ) {
        [[NSColor blackColor] set];
        NSRectFill([self bounds]);
        [self setSpotList:[self bounds]];
        [timer release];
        timer = [[NSDate dateWithTimeIntervalSinceNow:(60.0 * 5)] retain];
    }

    for (i = 0; i<numSpots; i++) {
        [[NSColor blackColor] set];
        NSRectFill(spots[i].rect);
    }
    for (i = 0; i<numSpots; i++) {
        [self moveSpot:i];
        [self drawSpot:i];
    }
}

- (void)drawSpot:(int)num
{
    NSRect r = spots[num].rect;
    NSBezierPath *circle = [NSBezierPath bezierPath];
    
    if (transparent) {
        [[NSColor colorWithCalibratedWhite:0.0 alpha:0.0] set];
        [circle appendBezierPathWithOvalInRect:InsetRect(spots[num].rect,2,2)];
        [circle fill];
        [buffer   compositeToPoint:r.origin fromRect:spots[num].rect operation:NSCompositeDestinationOver];
        
    } else {
        [[NSColor colorWithCalibratedWhite:1.0 alpha:1.0] set];
        [circle appendBezierPathWithOvalInRect:InsetRect(spots[num].rect,2,2)];
        [circle fill];
    }
}

- (void)moveSpot:(int)num
{
    spots[num].vector.x += (0.002 * speed); if (spots[num].vector.x > (2 * M_PI)) spots[num].vector.x -= (2 * M_PI);
    spots[num].vector.y += (0.002 * speed); if (spots[num].vector.y > (2 * M_PI)) spots[num].vector.y -= (2 * M_PI);

    spots[num].rect.origin.x = cos(spots[num].vector.x) * midx + midx;
    spots[num].rect.origin.y = sin(spots[num].vector.y) * midy + midy;


    /*NSPoint pt, v;
    float w, h;

    pt = spots[num].rect.origin;
    v =  spots[num].vector;
    w = [self bounds].size.width - spotRect.size.width;
    h = [self bounds].size.height - spotRect.size.height;

    // if we're about to move the spot out of the window, reverse the sign on the vector so
    // we head back in again.
    if ( (( v.x + pt.x ) > w) || (( v.x + pt.x ) < 0) )
        v.x *= -1.0;
    if ( (( v.y + pt.y ) > h) || (( v.y + pt.y ) < 0) )
        v.y *= -1.0;
    pt.x += v.x;
    pt.y += v.y;

    spots[num].rect.origin = pt;
    spots[num].vector = v;*/
}

- (NSPoint)randomPointForSize:(NSSize)ssize within:(NSRect)bounds
{
    NSPoint pt;

    pt.x = floor(randBetween(bounds.origin.x, bounds.size.width - ssize.width));
    pt.y = floor(randBetween(bounds.origin.y, bounds.size.height - ssize.height));

    return pt;
}


- (void)enteredScreenSaverMode
{
    [self setSpotList:[self bounds]];
}

- (void)willEnterScreenSaverMode
{
    if (transparent) [self loadBuffer];
}

- (void)willExitScreenSaverMode
{
    if (buffer) {
        [buffer release];
        buffer = nil;
    }
    [timer release];
    timer = nil;
}

// this will probably break in Mac OS X.
- (void)loadBuffer
{
    id snapShot;
    NSSize screen;
    NSRect imageRect = NSMakeRect(0,0,0,0);

    screen = [self bounds].size;
    imageRect.size = screen;
    snapShot = [[NSWindow alloc] initWithContentRect:imageRect
                                           styleMask:NSBorderlessWindowMask
                                             backing:NSBackingStoreNonretained
                                               defer:NO];
    PSsetwindowlevel(NSPopUpMenuWindowLevel,[snapShot windowNumber]);
    PSsetautofill(NO,[snapShot windowNumber]);
    [snapShot orderFront:self];
    if (!buffer) {
            buffer = [[NSImage alloc] initWithSize:screen];
    }
    [buffer lockFocus];
    PScomposite(0.0,0.0,screen.width,screen.height,[snapShot gState],
                0.0,0.0,NSCompositeCopy);
    [buffer unlockFocus];
    [snapShot release];
}

//
// User Interface
//

- (void)sizeSet:(id)sender
{
   
    size = [sizeSlider intValue];
    [sizeText setStringValue:sizeStrings[size-1]];

    spotRect = NSMakeRect(0,0,40.0 * size, 40.0 * size);
    [self setSpotList:[self bounds]];

    [[NSUserDefaults standardUserDefaults] setObject:[NSString stringWithFormat:@"%d",size] forKey:@"SpotLightSize"];
}

- (void)speedSet:(id)sender
{
    speed = [speedSlider intValue];
    [speedText setStringValue:speedStrings[speed-1]];

    [self setSpotList:[self bounds]];

    [[NSUserDefaults standardUserDefaults] setObject:[NSString stringWithFormat:@"%d",speed] forKey:@"SpotLightSpeed"];
}

- (void)spotSet:(id)sender
{
    numSpots = [spotSlider intValue];
    [spotText setStringValue:[NSString stringWithFormat:@"%d", numSpots]];

    [self setSpotList:[self bounds]];

    [[NSUserDefaults standardUserDefaults] setObject:[NSString stringWithFormat:@"%d",numSpots] forKey:@"SpotLightSpots"];
}

- (void)transparentSet:(id)sender
{
    transparent = [transparentBox state];
    [[NSUserDefaults standardUserDefaults] setObject:(transparent?@"YES":@"NO") forKey:@"SpotLightTrans"];
}

@end

@implementation StaticSpotLightView

- (id)initWithFrame:(NSRect)frameRect
{
    NSRect r = frameRect;
    int i, saftey=0;
    BOOL visible = NO;
    
    if (![super initWithFrame:frameRect loadInspectorPanel:NO]) return nil;

    r = NSMakeRect(r.origin.x - (r.size.width/2),
                   r.origin.y - (r.size.height/2),
                   r.size.width * 2,
                   r.size.height* 2);

    numSpots = 3;
    size = 3;
    spotRect = NSMakeRect(0,0,30.0 * size, 30.0 * size);

    // make sure something is visible in the static view;
    // give up if we can't figure it out in 100 tries.
    do {
        [self setSpotList:r];
        
        for (i = 0; i < numSpots; i++) {
            if (!NSIsEmptyRect(NSIntersectionRect([self bounds] , spots[i].rect))) {
                visible = YES;
                break;
            }
        }

        if (saftey++ > 100) break;
    } while(!visible);

    return self;
}

- (void)drawRect:(NSRect)rects
{
    int i;

    // clip to our rect.
    NSRectClip(rects);

    // do the drawing, which is now clipped to 'rects'
    [[NSColor blackColor] set];
    NSRectFill(rects);
    for (i = 0; i < numSpots; i++) {
        [self drawSpot:i];
    }
}

- (void)drawSpot:(int)num
{
    NSRect r = spots[num].rect;
    NSBezierPath *circle = [NSBezierPath bezierPath];

    [[NSColor grayColor] set];
    [circle appendBezierPathWithOvalInRect:r];
    [circle fill];
}


@end
