/*
 * YumView.m
 * 2/20/99 Mike Trent
 *
 * Yum screen saver module. Inspired by the Yum poster and the Colors TV
 * commercial.
 */

#import "YumView.h"

#ifdef GNUSTEP
#import <math.h>
#import <time.h>
#endif

static double kWedgeAngle = 0.0;

@implementation YumView

//
// Initialization
//

+ (void)initialize
{
    kWedgeAngle = (2.0 * M_PI) / 5.0;
}

- (BOOL)useBufferedWindow
{
    return YES;
}

- (BOOL)useMultipleMonitors
{
    return NO;
}

- (id)initWithFrame:(NSRect)frameRect
{
    NSMutableArray *list;
    NSBundle *bundle;
    
    self = [super initWithFrame:frameRect];

    if (self) {
        [self allocateGState];
        srandom(time(0));

        // load graphics
        list = [NSMutableArray array];
        bundle = [NSBundle bundleForClass:[self class]];
        [list addObject:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"blueberry48.tiff"]] autorelease]];
        [list addObject:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"grape48.tiff"]] autorelease]];
        [list addObject:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"tangerine48.tiff"]] autorelease]];
        [list addObject:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"lime48.tiff"]] autorelease]];
        [list addObject:[[[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"strawberry48.tiff"]] autorelease]];
        _icons = [[NSArray arrayWithArray:list] retain];
    }

    return self;
}

- (NSView *)inspector:sender
{
    
    if (!sharedInspectorPanel) {
        [NSBundle loadNibNamed:@"Yum.nib" owner:self];
    }

    return sharedInspectorPanel;
}

//
// The Graphics Code
//

- (void)setFrameSize:(NSSize)newSize
{
    [super setFrameSize:newSize];

    // set default state.
    _state.theta = M_PI * 1.5;
    _state.distance = 100.0;
    _state.delta = -0.025;

    // set our initial action
    if (_action) [_action release];
    _action = [[YumStartAction alloc] initWithState:&_state];
}

- (void)drawRect:(NSRect)rects
{
    [[NSColor blackColor] set];
    NSRectFill(rects);
}

- (void)oneStep
{
    int i;

   // erase our icons
    for (i = 0; i < 5; i++) { // MDT remove constants
        [[NSColor blackColor] set];
        NSRectFill([self rectForNumber:i]);
    }

    // adjust our state
    [_action changeState:&_state];
    _state.theta += _state.delta;
    if (_state.theta > (2.0 * M_PI)) _state.theta -= (2.0 * M_PI);
    if (_state.theta < 0.0) _state.theta += (2.0 * M_PI);

    // draw our icons
    for (i = 0; i < 5; i++) { // MDT remove constants
        [[_icons objectAtIndex:i] compositeToPoint:[self pointForNumber:i]
                                         operation:NSCompositeCopy];
    }

    // create new action if necessary
    if ([_action expired]) {
        _action = [_action newActionForState:&_state];
    }
}

- (NSRect)rectForNumber:(int)number
{
    NSRect r;

    r.origin = [self pointForNumber:number];
    r.size = NSMakeSize(48, 48);
    return r;
}

- (NSPoint)pointForNumber:(int)number
{
    float theta = _state.theta - (number * kWedgeAngle);
    float cost = cos(theta);
    float sint = sin(theta);
    NSRect r = [self bounds];

    return NSMakePoint(floor((r.size.width - 48) / 2 + (_state.distance * cost)),
                       floor((r.size.height - 48) / 2 + (_state.distance * sint)));
}

@end

@implementation StaticYumView

// in this implementation of draw rect, we want to arrange our icons as they
// live on the Yum poster. The image should also be off center so it better
// fills the black space right of the "Yum" title on our inspector panel.
// I use the x fudge factor to shift the image over. I use the y fudge factor
// to make the image fit on our view; I think our distance value is too large
// for the default view. I'm not sure what will happen if the inspector
// window becomes resizeable.
- (void)drawRect:(NSRect)rects
{
    int i;

    [super drawRect:rects];

    _state.theta = M_PI * 1.5;
    _state.distance = 50.0;
    for (i = 0; i < 5; i++) { // MDT remove constants
        NSPoint p = [self pointForNumber:i];
        p.y+=6; 
        p.x+=12; 
        [[_icons objectAtIndex:i] compositeToPoint:p
                                         operation:NSCompositeSourceOver];
    }
}

@end

